﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel;
using System.Diagnostics;
using System.Management.Automation;
using System.Management.Automation.Provider;
using System.Collections.ObjectModel;
using System.Xml.Linq;
using System.Xml.XPath;
using System.IO;



namespace PSModuleSample
{
    [CmdletProvider("Resource", ProviderCapabilities.ShouldProcess)]
    public class ResourceProvider : NavigationCmdletProvider
    {
        protected override bool IsValidPath(string path)
        {
            return true;
        }

        protected override bool IsItemContainer(string path)
        {
            var item = Item.CreateItem(path);
            return item.IsContainer;
        }

        protected override bool ItemExists(string path)
        {
            var item = Item.CreateItem(path);
            return (item != null);
        }

        protected override bool HasChildItems(string path)
        {
            var item = Item.CreateItem(path);
            return item.IsContainer;
        }

        protected override Collection<PSDriveInfo> InitializeDefaultDrives()
        {
            var drives = new Collection<PSDriveInfo>();
            drives.Add(
                new PSDriveInfo("Res", base.ProviderInfo, "", "", PSCredential.Empty)
            );
            return drives;
        }

        protected override void GetChildItems(string path, bool recurse)
        {
            var item = Item.CreateItem(path);

            if (item.IsContainer)
            {
                foreach (var child in ((Location)item).ChildItems)
                {
                    WriteItemObject(child, child.Path, child.IsContainer);
                }
            }
        }

        protected override string GetChildName(string path)
        {
            return Path.GetFileName(path);
        }

        protected override void GetChildNames(string path, ReturnContainers returnContainers)
        {
            var item = Item.CreateItem(path);

            if (item.IsContainer)
            {
                foreach (var child in ((Location)item).ChildItems)
                {
                     WriteItemObject(child.Name, child.Path,false);
                }
            }
        }

        protected override void GetItem(string path)
        {
            var item = Item.CreateItem(path);
            WriteItemObject(item, path, item.IsContainer);
        }

    }

    public abstract class Item
    {
        public string Name { get; set; }
        public string Value { get; set; }
        public string Path { get; set; }
        public bool IsContainer { get; set; }

        public Item(string path)
        {
            Name = System.IO.Path.GetFileName(path);
            Path = path;
        }

        public override string ToString()
        {
            return Name;
        }

        public static Item CreateItem(string path)
        {
            Item item;

            var names = path.Split('\\');
            var xpathBuilder=new StringBuilder();
            xpathBuilder.Append("/Root");

            for (var i = 0; i < names.Length - 1; i++)
            {
                xpathBuilder.Append("/Location[@Name = \"").Append(names[i]).Append("\"]");
            }

            var xpath = xpathBuilder.ToString();

            XElement location;
            if (path == "")
            {
                location = Resource.Instance.Xml.Root;
            }
            else
            {
                location = Resource.Instance.Xml.XPathSelectElement(string.Format("{0}/Location[@Name = \"{1}\"]", xpath, names.Last()));
            }
            
            if (location != null)
            {
                item = new Location(path);

                ((Location)item).ChildItems.AddRange(
                    location.XPathSelectElements("Location").Select(x => new Location(string.Format("{0}\\{1}", path, x.Attribute("Name").Value))));
                ((Location)item).ChildItems.AddRange(
                    location.XPathSelectElements("Setting").Select(x=>new Setting(string.Format("{0}\\{1}", path, x.Attribute("Name").Value), x.Attribute("Value").Value)));
                return item;
            }

            var setting = Resource.Instance.Xml.XPathSelectElement(string.Format("{0}/Setting[@Name = \"{1}\"]", xpath, names.Last()));

            if (setting != null)
            {
                item = new Setting(path, setting.Attribute("Value").Value);
                return item;
            }
            return null;
        }
    }

    public class Location : Item
    {

        public Location(string path)
            : base(path)
        {
            IsContainer = true;
            ChildItems = new List<Item>();
        }

        public List<Item> ChildItems
        {
            get;set;
        }

    }

    public class Setting : Item
    {
        public Setting(string path,string value)
            : base(path)

        {
            Value = value;
        }
    }

    public sealed class Resource
    {
        public static readonly Resource Instance = new Resource();

        private XDocument _xml;

        public XDocument Xml
        {
            get { return _xml; }
            set { _xml = value; }
        }

        private Resource()
        {
            var str = @"<Root>
<Location Name='configurations'>
  <Location Name='configuration'>
    <Location Name='UserList'>
      <Setting Name='User1' Value='test01' />
      <Setting Name='User2' Value='test02' />
      <Setting Name='User3' Value='test03' />
    </Location>
    <Setting Name='ComputerName' Value='computer01' />
    <Setting Name='IPAddress' Value='192.168.0.43' />
  </Location>
</Location>
</Root>";
            _xml = XDocument.Parse(str);
        }
    }
}
